import base64
import xml.etree.ElementTree as ET
import zlib
from typing import NamedTuple

from PoBExporter._fetch import (
    ascendancy_class_to_ascendancy_class_id,
    ascendancy_class_to_base_class,
    base_class_to_class_id,
    coordinate_to_jewel_hash,
)
from PoBExporter._item import get_item_text, get_slot, get_slots, set_gems
from PoBExporter._passive_encode import get_passive_tree_url
from PoBExporter._schema import Character


class PoBReturn(NamedTuple):
    pob_string: str
    primary_skill_name: str


def create_pob_string(character: Character) -> PoBReturn:
    if character["class"] in ascendancy_class_to_base_class:
        base_class = ascendancy_class_to_base_class[character["class"]]
        ascendant_class = character["class"]
        class_id = base_class_to_class_id[base_class]
        ascendancy_class_id = ascendancy_class_to_ascendancy_class_id[ascendant_class]
    else:
        base_class = character["class"]
        ascendant_class = "None"
        class_id = base_class_to_class_id[base_class]
        ascendancy_class_id = "nil"

    root = ET.Element("PathOfBuilding")
    build = ET.SubElement(
        root,
        "Build",
        level=str(character["level"]),
        targetVersion="3_0",
        pantheonMajorGod=character["passives"].get("pantheon_major") or "None",
        pantheonMinorGod=character["passives"].get("pantheon_minor") or "None",
        bandit=character["passives"].get("bandit_choice") or "None",
        className=base_class,
        ascendClassName=ascendant_class,
        characterLevelAutoMode="false",
        mainSocketGroup="1",
        viewMode="IMPORT",
    )
    ET.SubElement(
        build, "TimelessData",
        devotionVariant2="1",
        searchListFallback="",
        searchList="",
        devotionVariant1="1"
    )
    tree = ET.SubElement(root, "Tree", activeSpec="1")
    spec = ET.SubElement(
        tree,
        "Spec",
        masteryEffects=",".join(
            [f"{{{k},{v}}}" for k, v in character["passives"]["mastery_effects"].items()]),
        nodes=",".join([str(x) for x in character["passives"]["hashes"]]),
        classId=class_id,
        ascendClassId=ascendancy_class_id,
        secondaryAscendClassId="nil",
        treeVersion="_".join(character["metadata"]["version"].split(".")[:2]),
    )
    ET.SubElement(spec, "URL").text = get_passive_tree_url(int(class_id), 0 if ascendancy_class_id == "nil" else int(ascendancy_class_id),
                                                           character["passives"]["hashes"], character["passives"]["mastery_effects"])
    jewel_sockets = ET.SubElement(spec, "Sockets")
    items = ET.SubElement(root, "Items", activeItemSet="1",
                          showStatDifferences="true", useSecondWeaponSet="nil")
    item_set = ET.SubElement(
        items, "ItemSet", useSecondWeaponSet="nil", id="1")

    item_slots = get_slots()
    item_id = 1
    for jewel in character["jewels"]:
        node_id = coordinate_to_jewel_hash[jewel["x"]]
        ET.SubElement(items, "Item", id=str(item_id)
                      ).text = get_item_text(jewel)
        ET.SubElement(jewel_sockets, "Socket",
                      nodeId=node_id, itemId=str(item_id))
        ET.SubElement(item_set, "SocketIdURL",
                      nodeId=node_id, name=f"Jewel {node_id}")
        item_id += 1

    primary_skill_name = set_gems(character, root)

    for item in character["equipment"]:
        if item["inventoryId"] in ["Weapon2", "Offhand2"]:
            continue

        ET.SubElement(items, "Item", id=str(item_id)
                      ).text = get_item_text(item)
        item_slots[get_slot(item)] = str(item_id)
        item_id += 1

    for slot, item_id in item_slots.items():
        ET.SubElement(item_set, "Slot", name=slot, itemId=item_id)

    ET.SubElement(root, "TreeView", searchStr="",
                  zoomY="0", zoomLevel="3", showStatDifferences="true", zoomX="0")
    pob_string = base64.urlsafe_b64encode(
        zlib.compress(ET.tostring(root))).decode('utf-8')
    return PoBReturn(pob_string, primary_skill_name)



import base64

from PoBExporter._fetch import skilltree


def get_passive_tree_url(classId: int, ascendClassId: int, allocNodes: list[int], masterySelections: dict[str, int]) -> str:
    data = [0, 0, 0, 6, classId,  ascendClassId]
    node_count = 0
    cluster_count = 0
    mastery_count = 0

    cluster_node_ids = []
    mastery_node_ids = []
    nodes = []

    for id in allocNodes:
        node = skilltree["nodes"][str(id)]
        if "classStartIndex" in node or "isAscendancyStart" in node:
            continue
        if id < 65536 and node_count < 255:
            nodes.append(id // 256)
            nodes.append(id % 256)
            node_count += 1
            if id in masterySelections:
                effect_id = int(masterySelections[id])
                mastery_node_ids.append(effect_id // 256)
                mastery_node_ids.append(effect_id % 256)
                mastery_node_ids.append(id // 256)
                mastery_node_ids.append(id % 256)
                mastery_count += 1
        elif id >= 65536:
            cluster_id = id - 65536
            cluster_node_ids.append(cluster_id // 256)
            cluster_node_ids.append(cluster_id % 256)
            cluster_count += 1

    data.append(node_count)
    data.extend(nodes)

    data.append(cluster_count)
    data.extend(cluster_node_ids)

    data.append(mastery_count)
    data.extend(mastery_node_ids)
    encoded_data = base64.urlsafe_b64encode(bytes(data)).decode('utf-8')
    final_data = encoded_data.replace("+", "-").replace("/", "_")
    return "https://www.pathofexile.com/passive-skill-tree/" + final_data



import xml.etree.ElementTree as ET

from PoBExporter._fetch import gem_data
from PoBExporter._schema import Character, Item


def get_item_text(item: Item) -> str:
    quality = ""
    sockets = ""
    level_req = ""
    limit = ""
    radius = ""

    for property in item.get("properties", []):
        if property["name"] == "Quality":
            quality = property["values"][0][0].replace(
                "+", "").replace("%", "")
        if property["name"] == "LevelReq":
            level_req = property["values"][0][0]
        if property["name"] == "Limited to":
            limit = property["values"][0][0]
        if property["name"] == "Radius":
            radius = property["values"][0][0]
    socket_group = 0
    sockets = []
    for socket_group in {socket["group"] for socket in item.get("sockets", [])}:
        sockets.append("-".join([socket["sColour"][0]
                                 for socket in item.get("sockets", []) if socket["group"] == socket_group]))
    sockets = " ".join(sockets)

    item_text = []
    item_text.append(f"Rarity: {item['rarity'].upper()}")
    if item["name"]:
        item_text.append(item["name"])
    if item["typeLine"]:
        item_text.append(item["typeLine"])
    item_text.append(f"Item Level: {item['ilvl']}")
    if quality:
        item_text.append(f"Quality: {quality}")
    if sockets:
        item_text.append(f"Sockets: {sockets}")
    if level_req:
        item_text.append(f"LevelReq: {level_req}")
    if limit:
        item_text.append(f"Limited to: {limit}")
    if radius:
        item_text.append(f"Radius: {radius}")
    item_text.append(
        f"Implicits: {len(item.get('implicitMods', []) + item.get('enchantMods', []))}")
    item_text.extend(item.get("implicitMods", []))
    item_text.extend(
        ["{crafted}" + mod for mod in item.get("enchantMods", [])])
    item_text.extend(
        ["{fractured}" + mod for mod in item.get("fracturedMods", [])])
    item_text.extend(item.get("explicitMods", []))
    item_text.extend(
        ["{crafted}" + mod for mod in item.get("craftedMods", [])])
    return "\n".join(item_text)


def get_gem_groups(item: Item) -> list[tuple[list[dict], str]]:
    sockets: list[list[int]] = []
    for socket_group in {socket["group"] for socket in item.get("sockets", [])}:
        group_ids = []
        for idx, socket in enumerate(item.get("sockets", [])):
            if socket["group"] == socket_group:
                group_ids.append(idx)
        sockets.append(group_ids)
    socketed_items = item.get("socketedItems", [])
    gem_groups = []
    for socket_group in sockets:
        gem_stats = []
        for socket_id in socket_group:
            if len(socketed_items) > socket_id:
                gem = socketed_items[socket_id]
                if "abyssJewel" in gem:
                    continue
                gem_stats.append(get_gem_stats(gem))
        gem_groups.append((gem_stats, get_slot(item)))
    return gem_groups


def set_gems(character: Character, root: ET.Element):
    gem_groups: list[tuple[list[dict], str]] = []
    for item in character["equipment"]:
        gem_groups.extend(get_gem_groups(item))
    skills = ET.SubElement(root, "Skills",
                           sortGemsByDPSField="CombinedDPS",
                           activeSkillSet="1",
                           sortGemsByDPS="true",
                           defaultGemQuality="0",
                           defaultGemLevel="normalMaximum",
                           showSupportGemTypes="ALL",
                           showAltQualityGems="false")

    if len(gem_groups) == 0:
        return ""

    max_num_supports = 0
    primary_skill_id = -1
    primary_skill_name = ""
    for idx, (gems, _) in enumerate(gem_groups):
        num_supports = sum(gem["support"] for gem in gems)
        skill_gems = [gem for gem in gems if not gem["support"]]
        if num_supports > max_num_supports and len(skill_gems) > 0:
            max_num_supports = num_supports
            primary_skill_id = idx
            primary_skill_name = skill_gems[0]["nameSpec"]

    skillset = ET.SubElement(skills, "SkillSet", id="1")

    gem_groups.insert(0, gem_groups.pop(primary_skill_id))
    for idx, (gems, slot) in enumerate(gem_groups):
        skill = ET.SubElement(skillset, "Skill", {
            "mainActiveSkillCalcs": "nil",
            "includeInFullDPS": "nil",
            "label": "",
            "enabled": "true",
            "slot": slot
        })

        if idx == 0:
            skill.set("mainActiveSkill", "1")
            skill.set("mainActiveSkillCalcs", "1")
            skill.set("includeInFullDPS", "true")

        for gem in gems:
            gem.pop("support")
            ET.SubElement(skill, "Gem", gem)
    return primary_skill_name


def get_gem_stats(gem: Item) -> dict:
    level = "1"
    quality = "0"
    for property in gem.get("properties", []):
        if property["name"] == "Quality":
            quality = property["values"][0][0].replace(
                "+", "").replace("%", "")
        if property["name"] == "Level":
            level = property["values"][0][0].replace(" (Max)", "")
    gem_info = gem_data.get(gem["baseType"], {})

    gemId = gem_info.get("gemId")
    variantId = gem_info.get("variantId")
    return {
        "level": level,
        "quality": quality,
        "gemId": gemId,
        "variantId": variantId,
        "qualityId": "Default",
        "enabled": "true",
        "count": "nil",
        "nameSpec": gem["baseType"],
        "support": gem.get("support", "false")
    }


def get_slot(item: Item) -> str:
    if item["inventoryId"] == "Flask":
        return f"Flask {item['x'] + 1}"
    return {
        "Weapon": "Weapon 1",
        "Offhand": "Weapon 2",
        "BodyArmour": "Body Armour",
        "Helm": "Helmet",
        "Gloves": "Gloves",
        "Boots": "Boots",
        "Ring": "Ring 1",
        "Ring2": "Ring 2",
        "Amulet": "Amulet",
        "Belt": "Belt",
    }.get(item["inventoryId"], item["inventoryId"])


def get_slots():
    return {
        # "Boots Abyssal Socket 5": "0",
        # "Belt Abyssal Socket 2": "0",
        # "Weapon 1 Swap Abyssal Socket 4": "0",
        # "Weapon 1 Abyssal Socket 5": "0",
        # "Weapon 2 Abyssal Socket 5": "0",
        # "Weapon 1 Swap Abyssal Socket 6": "0",
        # "Weapon 1 Abyssal Socket 6": "0",
        # "Weapon 2 Abyssal Socket 6": "0",
        # "Weapon 2 Swap Abyssal Socket 1": "0",
        # "Boots Abyssal Socket 3": "0",
        # "Weapon 1 Abyssal Socket 2": "0",
        # "Weapon 2 Swap Abyssal Socket 4": "0",
        # "Helmet Abyssal Socket 1": "0",
        # "Belt Abyssal Socket 6": "0",
        # "Helmet Abyssal Socket 2": "0",
        # "Helmet Abyssal Socket 3": "0",
        # "Weapon 2 Swap Abyssal Socket 5": "0",
        # "Weapon 1 Abyssal Socket 1": "0",
        # "Weapon 2 Abyssal Socket 1": "0",
        # "Weapon 1 Abyssal Socket 4": "0",
        # "Weapon 1 Swap Abyssal Socket 2": "0",
        # "Boots Abyssal Socket 4": "0",
        # "Weapon 2 Abyssal Socket 2": "0",
        # "Weapon 1 Swap Abyssal Socket 3": "0",
        # "Boots Abyssal Socket 2": "0",
        # "Belt Abyssal Socket 3": "0",
        # "Body Armour Abyssal Socket 1": "0",
        # "Gloves Abyssal Socket 1": "0",
        # "Helmet Abyssal Socket 5": "0",
        # "Helmet Abyssal Socket 6": "0",
        # "Weapon 1 Swap Abyssal Socket 1": "0",
        # "Weapon 2 Swap Abyssal Socket 2": "0",
        # "Helmet Abyssal Socket 4": "0",
        # "Gloves Abyssal Socket 5": "0",
        # "Body Armour Abyssal Socket 2": "0",
        # "Weapon 1 Abyssal Socket 3": "0",
        # "Weapon 2 Abyssal Socket 3": "0",
        # "Belt Abyssal Socket 4": "0",
        # "Belt Abyssal Socket 1": "0",
        # "Body Armour Abyssal Socket 3": "0",
        # "Weapon 2 Swap": "0",
        # "Belt Abyssal Socket 5": "0",
        # "Boots Abyssal Socket 1": "0",
        # "Body Armour Abyssal Socket 4": "0",
        # "Weapon 2 Swap Abyssal Socket 6": "0",
        # "Weapon 2 Swap Abyssal Socket 3": "0",
        # "Weapon 1 Swap": "0",
        # "Gloves Abyssal Socket 2": "0",
        # "Body Armour Abyssal Socket 6": "0",
        # "Weapon 1 Swap Abyssal Socket 5": "0",
        # "Weapon 2 Abyssal Socket 4": "0",
        # "Gloves Abyssal Socket 3": "0",
        # "Body Armour Abyssal Socket 5": "0",
        # "Gloves Abyssal Socket 4": "0",
        # "Gloves Abyssal Socket 6": "0",
        # "Boots Abyssal Socket 6": "0",
        "Gloves": "0",
        "Weapon 1": "0",
        "Flask 3": "0",
        "Helmet": "0",
        "Belt": "0",
        "Flask 5": "0",
        "Flask 1": "0",
        "Amulet": "0",
        "Flask 4": "0",
        "Flask 2": "0",
        "Weapon 2": "0",
        "Ring 2": "0",
        "Body Armour": "0",
        "Ring 1": "0",
        "Boots": "0",
    }



from typing import NotRequired, TypedDict, Union


class LeagueCategory(TypedDict):
    id: str
    current: NotRequired[bool]


class LeagueRule(TypedDict):
    id: str
    name: str
    description: NotRequired[str]


class League(TypedDict):
    id: str
    realm: NotRequired[str]
    description: NotRequired[str]
    category: NotRequired[LeagueCategory]
    rules: NotRequired[list[LeagueRule]]
    registerAt: NotRequired[str]
    event: NotRequired[bool]
    url: NotRequired[str]
    startAt: NotRequired[str]
    endAt: NotRequired[str]
    timedEvent: NotRequired[bool]
    scoreEvent: NotRequired[bool]
    delveEvent: NotRequired[bool]
    ancestorEvent: NotRequired[bool]
    leagueEvent: NotRequired[bool]


class Depth(TypedDict):
    default: NotRequired[int]
    solo: NotRequired[int]


Character = TypedDict('Character', {
    "id": str,
    "name": str,
    "realm": str,
    "class": str,
    "league": NotRequired[str],
    "level": int,
    "experience": int,
    "ruthless": NotRequired[bool],
    "expired": NotRequired[bool],
    "deleted": NotRequired[bool],
    "current": NotRequired[bool],
    "equipment": list['Item'],
    "inventory": list['Item'],
    "rucksack": NotRequired[list['Item']],
    "jewels": list['Item'],
    "passives": 'Passives',
    "metadata": 'CharacterMetadata'
})


class LadderEntry(TypedDict):
    rank: int
    dead: NotRequired[bool]
    retired: NotRequired[bool]
    ineligible: NotRequired[bool]
    public: NotRequired[bool]
    character: Character
    account: NotRequired['Account']


class PrivateLeague(TypedDict):
    name: str
    url: str


class EventLadderEntry(TypedDict):
    rank: int
    ineligible: NotRequired[bool]
    time: NotRequired[int]
    private_league: PrivateLeague


class Guild(TypedDict):
    id: int
    name: str
    tag: str


class TwitchStream(TypedDict):
    name: str
    image: str
    status: str


class Twitch(TypedDict):
    name: str
    stream: NotRequired[TwitchStream]


class Challenges(TypedDict):
    set: str
    completed: int
    max: int


class Account(TypedDict):
    name: str
    realm: NotRequired[str]
    guild: NotRequired[Guild]
    challenges: NotRequired[Challenges]
    twitch: NotRequired[Twitch]


class PvPMatch(TypedDict):
    id: str
    realm: NotRequired[str]
    startAt: NotRequired[str]
    endAt: NotRequired[str]
    url: NotRequired[str]
    description: str
    glickoRatings: bool
    pvp: bool
    style: str
    registerAt: NotRequired[str]
    complete: NotRequired[bool]
    upcoming: NotRequired[bool]
    inProgress: NotRequired[bool]


class PvPLadderTeamMember(TypedDict):
    account: Account
    character: Character
    public: NotRequired[bool]


class PvPLadderTeamEntry(TypedDict):
    rank: int
    rating: NotRequired[int]
    points: NotRequired[int]
    games_played: NotRequired[int]
    cumulative_opponent_points: NotRequired[int]
    last_game_time: NotRequired[str]
    members: list[PvPLadderTeamMember]


class ItemSocket(TypedDict):
    group: int
    attr: str
    sColour: str


class ItemProperty(TypedDict):
    name: str
    values: list[tuple[str, int]]
    displayMode: NotRequired[int]
    progress: NotRequired[float]
    type: NotRequired[int]
    suffix: NotRequired[str]


class LogbookFaction(TypedDict):
    id: str
    name: str


class LogbookMod(TypedDict):
    name: str
    faction: LogbookFaction


class ItemReward(TypedDict):
    label: str
    rewards: dict[str, int]


class IncubatedItem(TypedDict):
    name: str
    level: int
    progress: int
    total: int


class Crucible(TypedDict):
    layout: str
    nodes: dict[str, 'CrucibleNode']


class Scourged(TypedDict):
    tier: int
    level: NotRequired[int]
    progress: NotRequired[int]
    total: NotRequired[int]


class UltimatumMod(TypedDict):
    type: str
    tier: int


class Hybrid(TypedDict):
    isVaalGem: NotRequired[bool]
    baseTypeName: str
    properties: NotRequired[list[ItemProperty]]
    explicitMods: NotRequired[list[str]]
    secDescrText: NotRequired[str]


class Extended(TypedDict):
    category: NotRequired[str]
    subcategories: NotRequired[list[str]]
    prefixes: NotRequired[int]
    suffixes: NotRequired[int]


class Item(TypedDict):
    verified: bool
    w: int
    h: int
    icon: str
    support: NotRequired[bool]
    stackSize: NotRequired[int]
    maxStackSize: NotRequired[int]
    stackSizeText: NotRequired[str]
    league: NotRequired[str]
    id: str
    influences: NotRequired[dict]
    elder: NotRequired[bool]
    shaper: NotRequired[bool]
    searing: NotRequired[bool]
    tangled: NotRequired[bool]
    abyssJewel: NotRequired[bool]
    delve: NotRequired[bool]
    fractured: NotRequired[bool]
    synthesised: NotRequired[bool]
    sockets: NotRequired[list[ItemSocket]]
    socketedItems: NotRequired[list['Item']]
    name: str
    typeLine: str
    baseType: str
    rarity: str
    identified: bool
    itemLevel: NotRequired[int]
    ilvl: int
    note: NotRequired[str]
    forum_note: NotRequired[str]
    lockedToCharacter: NotRequired[bool]
    lockedToAccount: NotRequired[bool]
    duplicated: NotRequired[bool]
    split: NotRequired[bool]
    corrupted: NotRequired[bool]
    unmodifiable: NotRequired[bool]
    cisRaceReward: NotRequired[bool]
    seaRaceReward: NotRequired[bool]
    thRaceReward: NotRequired[bool]
    properties: NotRequired[list[ItemProperty]]
    notableProperties: NotRequired[list[ItemProperty]]
    requirements: NotRequired[list[ItemProperty]]
    additionalProperties: NotRequired[list[ItemProperty]]
    nextLevelRequirements: NotRequired[list[ItemProperty]]
    talismanTier: NotRequired[int]
    rewards: NotRequired[list[ItemReward]]
    secDescrText: NotRequired[str]
    utilityMods: NotRequired[list[str]]
    logbookMods: NotRequired[list[LogbookMod]]
    enchantMods: NotRequired[list[str]]
    scourgeMods: NotRequired[list[str]]
    implicitMods: NotRequired[list[str]]
    ultimatumMods: NotRequired[list[UltimatumMod]]
    explicitMods: NotRequired[list[str]]
    craftedMods: NotRequired[list[str]]
    fracturedMods: NotRequired[list[str]]
    crucibleMods: NotRequired[list[str]]
    cosmeticMods: NotRequired[list[str]]
    veiledMods: NotRequired[list[str]]
    veiled: NotRequired[bool]
    descrText: NotRequired[str]
    flavourText: NotRequired[list[str]]
    flavourTextParsed: NotRequired[list[Union[str, dict]]]
    flavourTextNote: NotRequired[str]
    prophecyText: NotRequired[str]
    isRelic: NotRequired[bool]
    foilVariation: NotRequired[int]
    replica: NotRequired[bool]
    foreseeing: NotRequired[bool]
    incubatedItem: NotRequired[IncubatedItem]
    scourged: NotRequired[Scourged]
    crucible: NotRequired[Crucible]
    ruthless: NotRequired[bool]
    frameType: NotRequired[int]
    artFilename: NotRequired[str]
    hybrid: NotRequired[Hybrid]
    extended: NotRequired[Extended]
    x: int
    y: int
    inventoryId: str
    socket: NotRequired[int]
    colour: NotRequired[str]


class PublicStashChange(TypedDict):
    id: str
    public: bool
    accountName: NotRequired[str]
    stash: NotRequired[str]
    lastCharacterName: NotRequired[str]
    stashType: str
    league: NotRequired[str]
    items: list[Item]


CrucibleNode = TypedDict('CrucibleNode', {
    "skill": NotRequired[int],
    "tier": NotRequired[int],
    "icon": NotRequired[str],
    "allocated": NotRequired[bool],
    "isNotable": NotRequired[bool],
    "isReward": NotRequired[bool],
    "stats": NotRequired[list[str]],
    "reminderText": NotRequired[list[str]],
    "orbit": NotRequired[int],
    "orbitIndex": NotRequired[int],
    "out": list[str],
    "in": list[str]
})


class ExpansionJewel(TypedDict):
    size: NotRequired[int]
    index: NotRequired[int]
    proxy: NotRequired[int]
    parent: NotRequired[int]


class MasteryEffect(TypedDict):
    effect: int
    stats: list[str]
    reminderText: NotRequired[list[str]]


PassiveNode = TypedDict('PassiveNode', {
    "skill": NotRequired[int],
    "name": NotRequired[str],
    "icon": NotRequired[str],
    "isKeystone": NotRequired[bool],
    "isNotable": NotRequired[bool],
    "isMastery": NotRequired[bool],
    "inactiveIcon": NotRequired[str],
    "activeIcon": NotRequired[str],
    "activeEffectImage": NotRequired[str],
    "masteryEffects": NotRequired[list[MasteryEffect]],
    "isBlighted": NotRequired[bool],
    "isTattoo": NotRequired[bool],
    "isProxy": NotRequired[bool],
    "isJewelSocket": NotRequired[bool],
    "expansionJewel": NotRequired[ExpansionJewel],
    "recipe": NotRequired[list[str]],
    "grantedStrength": NotRequired[int],
    "grantedDexterity": NotRequired[int],
    "grantedIntelligence": NotRequired[int],
    "ascendancyName": NotRequired[str],
    "isAscendancyStart": NotRequired[bool],
    "isMultipleChoice": NotRequired[bool],
    "isMultipleChoiceOption": NotRequired[bool],
    "grantedPassivePoints": NotRequired[int],
    "stats": NotRequired[list[str]],
    "reminderText": NotRequired[list[str]],
    "flavourText": NotRequired[list[str]],
    "classStartIndex": NotRequired[int],
    "group": NotRequired[str],
    "orbit": NotRequired[int],
    "orbitIndex": NotRequired[int],
    "out": list[str],
    "in": list[str]
})


class PassiveGroup(TypedDict):
    x: float
    y: float
    orbits: list[int]
    isProxy: NotRequired[bool]
    proxy: NotRequired[str]
    nodes: list[str]


class Subgraph(TypedDict):
    groups: dict[str, PassiveGroup]
    nodes: dict[str, PassiveNode]


class ItemJewelData(TypedDict):
    type: str
    radius: NotRequired[int]
    radiusMin: NotRequired[int]
    radiusVisual: NotRequired[str]
    subgraph: NotRequired[Subgraph]


class CharacterMetadata(TypedDict):
    version: str


class Passives(TypedDict):
    hashes: list[int]
    hashes_ex: list[int]
    mastery_effects: dict[str, int]
    skill_overrides: NotRequired[dict[str, 'PassiveNode']]
    bandit_choice: NotRequired[str]
    pantheon_major: NotRequired[str]
    pantheon_minor: NotRequired[str]
    jewel_data: dict[str, ItemJewelData]
    alternate_ascendancy: NotRequired[str]


class StashTabMetaDict(TypedDict):
    public: NotRequired[bool]
    folder: NotRequired[bool]
    colour: NotRequired[str]


class StashTab(TypedDict):
    id: str
    parent: NotRequired[str]
    name: str
    type: str
    index: NotRequired[int]
    metadata: StashTabMetaDict
    children: NotRequired[list['StashTab']]
    items: NotRequired[list[Item]]


class AtlasTreeInfo(TypedDict):
    name: str
    hashess: list[int]


class AtlasTreeInfoOld(TypedDict):
    hashes: list[int]


class LeagueAccount(TypedDict):
    atlas_passives: NotRequired[AtlasTreeInfoOld]
    atlas_passive_trees: list[AtlasTreeInfo]


class ValidationDetails(TypedDict):
    valid: bool
    version: NotRequired[str]
    validated: NotRequired[str]


class ItemFilter(TypedDict):
    id: str
    filter_name: str
    realm: str
    description: str
    version: str
    type: str
    public: NotRequired[bool]
    filter: NotRequired[str]
    validation: NotRequired[ValidationDetails]



import json
import os
from typing import TypedDict

import requests

data_path = os.path.join(os.path.split(__file__)[0], "poe_data", "")


def get_gem_dict():
    resp = requests.get(
        "https://raw.githubusercontent.com/lvlvllvlvllvlvl/RePoE/refs/heads/master/RePoE/data/gems.json")
    gem_dict = {}
    for gem_id, gem_data in resp.json().items():

        if "Royale" in gem_id or "Playtest" in gem_id or gem_data["display_name"] is None or "UNUSED" in gem_data["display_name"]:
            continue
        base_item = gem_data["base_item"]
        if not base_item:
            continue
        gem_dict[gem_data["display_name"]] = {
            "variantId": gem_id,
            "gemId": gem_data["base_item"]["id"]
        }

    with open(os.path.join(data_path, "gems.json"), "w") as file:
        file.write(json.dumps(gem_dict, separators=(',', ':')))


def get_skilltree():
    resp = requests.get(
        "https://raw.githubusercontent.com/grindinggear/skilltree-export/master/data.json")
    with open(os.path.join(data_path, "skilltree.json"), "w") as file:
        file.write(json.dumps(resp.json(), separators=(',', ':')))


class GemData(TypedDict):
    gemId: str
    variantId: str


coordinate_to_jewel_hash: list[str] = []
gem_data: dict[str, GemData] = {}
skilltree: dict = {}
base_class_to_class_id: dict = {}
ascendancy_class_to_ascendancy_class_id: dict = {}
ascendancy_class_to_base_class: dict = {}


if __name__ == "__main__":
    if not os.path.exists(data_path):
        os.makedirs(data_path)
    get_gem_dict()
    get_skilltree()
else:

    with open(os.path.join(data_path, "gems.json"), "r") as file:
        gem_data: dict[str, GemData] = json.load(file)

    with open(os.path.join(data_path, "skilltree.json"), "r") as file:
        skilltree = json.load(file)

    coordinate_to_jewel_hash = [
        '26725', '36634', '33989', '41263', '60735', '61834', '31683', '28475', '6230', '48768', '34483', '7960',
        '46882', '55190', '61419', '2491', '54127', '32763', '26196', '33631', '21984', '29712', '48679', '9408',
        '12613', '16218', '2311', '22994', '40400', '46393', '61305', '12161', '3109', '49080', '17219', '44169',
        '24970', '36931', '14993', '10532', '23756', '46519', '23984', '51198', '61666', '6910', '49684', '33753',
        '18436', '11150', '22748', '64583', '61288', '13170', '9797', '41876', '59585',
    ]

    for class_id, base_class in enumerate(skilltree["classes"]):
        base_class_to_class_id[base_class["name"]] = str(class_id)
        for ascendancy_class_id, ascendancy_class in enumerate(base_class["ascendancies"]):
            ascendancy_class_to_ascendancy_class_id[ascendancy_class["name"]
                                                    ] = str(ascendancy_class_id + 1)
            ascendancy_class_to_base_class[ascendancy_class["name"]
                                           ] = base_class["name"]



import os
import sys

from setuptools import setup

# Package meta-data.
NAME = "PoBExporter"
DESCRIPTION = "Python library to generate Path of Building exports directly from PoE API without running PoB headless"
URL = "https://github.com/Liberatorist/PoBExporter"
EMAIL = ""
AUTHOR = "Liberatorist"
REQUIRES_PYTHON = ">=3.11.0"
VERSION = "3.25.14"

# What packages are required for this module to be executed?
REQUIRED = []

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['django'],
}
data_files = []
for file in os.listdir("PoBExporter/poe_data"):
    # not packing all stat translations for now
    data_files.append(os.path.join("PoBExporter/poe_data", file))
directory = "/".join(sys.prefix.split("/")[:-2]) + "/PoBExporter/poe_data"


setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    py_modules=["PoBExporter._create_export", "PoBExporter._item",
                "PoBExporter._passive_encode", "PoBExporter._schema", "PoBExporter._fetch"],
    install_requires=REQUIRED,
    extras_require=EXTRAS,
    license="proprietary",

    data_files=[
        (directory, data_files),
    ]

)
